"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPatchedDependency = getPatchedDependency;
exports.getVersionsFromLockfile = getVersionsFromLockfile;
const path_1 = __importDefault(require("path"));
const parse_wanted_dependency_1 = require("@pnpm/parse-wanted-dependency");
const enquirer_1 = require("enquirer");
const lockfile_fs_1 = require("@pnpm/lockfile.fs");
const lockfile_utils_1 = require("@pnpm/lockfile.utils");
const error_1 = require("@pnpm/error");
const modules_yaml_1 = require("@pnpm/modules-yaml");
const pick_fetcher_1 = require("@pnpm/pick-fetcher");
const realpath_missing_1 = __importDefault(require("realpath-missing"));
const semver_1 = __importDefault(require("semver"));
async function getPatchedDependency(rawDependency, opts) {
    const dep = (0, parse_wanted_dependency_1.parseWantedDependency)(rawDependency);
    const { versions, preferredVersions } = await getVersionsFromLockfile(dep, opts);
    if (!preferredVersions.length) {
        throw new error_1.PnpmError('PATCH_VERSION_NOT_FOUND', `Can not find ${rawDependency} in project ${opts.lockfileDir}, ${versions.length ? `you can specify currently installed version: ${versions.map(({ version }) => version).join(', ')}.` : `did you forget to install ${rawDependency}?`}`);
    }
    dep.alias = dep.alias ?? rawDependency;
    if (preferredVersions.length > 1) {
        const { version, applyToAll } = await (0, enquirer_1.prompt)([{
                type: 'select',
                name: 'version',
                message: 'Choose which version to patch',
                choices: preferredVersions.map(preferred => ({
                    name: preferred.version,
                    message: preferred.version,
                    value: preferred.gitTarballUrl ?? preferred.version,
                    hint: preferred.gitTarballUrl ? 'Git Hosted' : undefined,
                })),
                result(selected) {
                    const selectedVersion = preferredVersions.find(preferred => preferred.version === selected);
                    return selectedVersion.gitTarballUrl ?? selected;
                },
            }, {
                type: 'confirm',
                name: 'applyToAll',
                message: 'Apply this patch to all versions?',
            }]);
        return {
            ...dep,
            applyToAll,
            pref: version,
        };
    }
    else {
        const preferred = preferredVersions[0];
        return {
            ...dep,
            applyToAll: !dep.pref,
            pref: preferred.gitTarballUrl ?? preferred.version,
        };
    }
}
async function getVersionsFromLockfile(dep, opts) {
    const modulesDir = await (0, realpath_missing_1.default)(path_1.default.join(opts.lockfileDir, opts.modulesDir ?? 'node_modules'));
    const modules = await (0, modules_yaml_1.readModulesManifest)(modulesDir);
    const lockfile = (modules?.virtualStoreDir && await (0, lockfile_fs_1.readCurrentLockfile)(modules.virtualStoreDir, {
        ignoreIncompatible: true,
    })) ?? null;
    if (!lockfile) {
        throw new error_1.PnpmError('PATCH_NO_LOCKFILE', 'The modules directory is not ready for patching', {
            hint: 'Run pnpm install first',
        });
    }
    const pkgName = dep.alias && dep.pref ? dep.alias : (dep.pref ?? dep.alias);
    const versions = Object.entries(lockfile.packages ?? {})
        .map(([depPath, pkgSnapshot]) => {
        const tarball = pkgSnapshot.resolution?.tarball ?? '';
        return {
            ...(0, lockfile_utils_1.nameVerFromPkgSnapshot)(depPath, pkgSnapshot),
            gitTarballUrl: (0, pick_fetcher_1.isGitHostedPkgUrl)(tarball) ? tarball : undefined,
        };
    })
        .filter(({ name }) => name === pkgName);
    return {
        versions,
        preferredVersions: versions.filter(({ version }) => dep.alias && dep.pref ? semver_1.default.satisfies(version, dep.pref) : true),
    };
}
//# sourceMappingURL=getPatchedDependency.js.map