"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandNames = exports.shorthands = void 0;
exports.rcOptionsTypes = rcOptionsTypes;
exports.cliOptionsTypes = cliOptionsTypes;
exports.help = help;
exports.handler = handler;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const patching_apply_patch_1 = require("@pnpm/patching.apply-patch");
const cli_utils_1 = require("@pnpm/cli-utils");
const config_1 = require("@pnpm/config");
const pick_1 = __importDefault(require("ramda/src/pick"));
const render_help_1 = __importDefault(require("render-help"));
const chalk_1 = __importDefault(require("chalk"));
const terminal_link_1 = __importDefault(require("terminal-link"));
const error_1 = require("@pnpm/error");
const writePackage_1 = require("./writePackage");
const getEditDirPath_1 = require("./getEditDirPath");
const getPatchedDependency_1 = require("./getPatchedDependency");
const stateFile_1 = require("./stateFile");
const read_project_manifest_1 = require("@pnpm/read-project-manifest");
function rcOptionsTypes() {
    return (0, pick_1.default)([], config_1.types);
}
function cliOptionsTypes() {
    return { ...rcOptionsTypes(), 'edit-dir': String, 'ignore-existing': Boolean };
}
exports.shorthands = {
    d: '--edit-dir',
};
exports.commandNames = ['patch'];
function help() {
    return (0, render_help_1.default)({
        description: 'Prepare a package for patching',
        descriptionLists: [{
                title: 'Options',
                list: [
                    {
                        description: 'The package that needs to be modified will be extracted to this directory',
                        name: '--edit-dir',
                    },
                    {
                        description: 'Ignore existing patch files when patching',
                        name: '--ignore-existing',
                    },
                ],
            }],
        url: (0, cli_utils_1.docsUrl)('patch'),
        usages: ['pnpm patch <pkg name>@<version>'],
    });
}
async function handler(opts, params) {
    if (opts.editDir && fs_1.default.existsSync(opts.editDir) && fs_1.default.readdirSync(opts.editDir).length > 0) {
        throw new error_1.PnpmError('PATCH_EDIT_DIR_EXISTS', `The target directory already exists: '${opts.editDir}'`);
    }
    if (!params[0]) {
        throw new error_1.PnpmError('MISSING_PACKAGE_NAME', '`pnpm patch` requires the package name');
    }
    const lockfileDir = opts.lockfileDir ?? opts.dir ?? process.cwd();
    const patchedDep = await (0, getPatchedDependency_1.getPatchedDependency)(params[0], {
        lockfileDir,
        modulesDir: opts.modulesDir,
        virtualStoreDir: opts.virtualStoreDir,
    });
    const modulesDir = path_1.default.join(lockfileDir, opts.modulesDir ?? 'node_modules');
    const editDir = opts.editDir
        ? path_1.default.resolve(opts.dir, opts.editDir)
        : (0, getEditDirPath_1.getEditDirPath)(params[0], patchedDep, { modulesDir });
    if (fs_1.default.existsSync(editDir) && fs_1.default.readdirSync(editDir).length !== 0) {
        throw new error_1.PnpmError('EDIT_DIR_NOT_EMPTY', `The directory ${editDir} is not empty`, {
            hint: 'Either run `pnpm patch-commit` to commit or delete it then run `pnpm patch` to recreate it',
        });
    }
    await (0, writePackage_1.writePackage)(patchedDep, editDir, opts);
    (0, stateFile_1.writeEditDirState)({
        editDir,
        modulesDir: path_1.default.join(opts.dir, opts.modulesDir ?? 'node_modules'),
        patchedPkg: params[0],
        applyToAll: patchedDep.applyToAll,
    });
    if (!opts.ignoreExisting) {
        let rootProjectManifest = opts.rootProjectManifest;
        if (!opts.sharedWorkspaceLockfile) {
            const { manifest } = await (0, read_project_manifest_1.tryReadProjectManifest)(lockfileDir);
            if (manifest) {
                rootProjectManifest = manifest;
            }
        }
        if (rootProjectManifest?.pnpm?.patchedDependencies) {
            tryPatchWithExistingPatchFile({
                allowFailure: patchedDep.applyToAll,
                patchedDep,
                patchedDir: editDir,
                patchedDependencies: rootProjectManifest.pnpm.patchedDependencies,
                lockfileDir,
            });
        }
    }
    return `Patch: You can now edit the package at:

  ${(0, terminal_link_1.default)(chalk_1.default.blue(editDir), 'file://' + editDir)}

To commit your changes, run:

  ${chalk_1.default.green(`pnpm patch-commit '${editDir}'`)}

`;
}
function tryPatchWithExistingPatchFile({ allowFailure, patchedDep, patchedDir, patchedDependencies, lockfileDir, }) {
    if (!patchedDep.alias || !patchedDep.pref) {
        return;
    }
    const existingPatchFile = patchedDependencies[`${patchedDep.alias}@${patchedDep.pref}`];
    if (!existingPatchFile) {
        return;
    }
    const existingPatchFilePath = path_1.default.resolve(lockfileDir, existingPatchFile);
    if (!fs_1.default.existsSync(existingPatchFilePath)) {
        throw new error_1.PnpmError('PATCH_FILE_NOT_FOUND', `Unable to find patch file ${existingPatchFilePath}`);
    }
    (0, patching_apply_patch_1.applyPatchToDir)({ patchedDir, patchFilePath: existingPatchFilePath, allowFailure });
}
//# sourceMappingURL=patch.js.map