"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandNames = exports.rcOptionsTypes = void 0;
exports.cliOptionsTypes = cliOptionsTypes;
exports.help = help;
exports.handler = handler;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const config_1 = require("@pnpm/config");
const error_1 = require("@pnpm/error");
const fs_packlist_1 = require("@pnpm/fs.packlist");
const plugin_commands_installation_1 = require("@pnpm/plugin-commands-installation");
const read_package_json_1 = require("@pnpm/read-package-json");
const read_project_manifest_1 = require("@pnpm/read-project-manifest");
const fast_glob_1 = __importDefault(require("fast-glob"));
const normalize_path_1 = __importDefault(require("normalize-path"));
const pick_1 = __importDefault(require("ramda/src/pick"));
const equals_1 = __importDefault(require("ramda/src/equals"));
const safe_execa_1 = __importDefault(require("safe-execa"));
const escape_string_regexp_1 = __importDefault(require("escape-string-regexp"));
const make_empty_dir_1 = __importDefault(require("make-empty-dir"));
const render_help_1 = __importDefault(require("render-help"));
const tempy_1 = __importDefault(require("tempy"));
const writePackage_1 = require("./writePackage");
const parse_wanted_dependency_1 = require("@pnpm/parse-wanted-dependency");
const getPatchedDependency_1 = require("./getPatchedDependency");
const stateFile_1 = require("./stateFile");
exports.rcOptionsTypes = cliOptionsTypes;
function cliOptionsTypes() {
    return (0, pick_1.default)(['patches-dir'], config_1.types);
}
exports.commandNames = ['patch-commit'];
function help() {
    return (0, render_help_1.default)({
        description: 'Generate a patch out of a directory',
        descriptionLists: [{
                title: 'Options',
                list: [
                    {
                        description: 'The generated patch file will be saved to this directory',
                        name: '--patches-dir',
                    },
                ],
            }],
        url: (0, cli_utils_1.docsUrl)('patch-commit'),
        usages: ['pnpm patch-commit <patchDir>'],
    });
}
async function handler(opts, params) {
    const userDir = params[0];
    const lockfileDir = (opts.lockfileDir ?? opts.dir ?? process.cwd());
    const patchesDirName = (0, normalize_path_1.default)(path_1.default.normalize(opts.patchesDir ?? 'patches'));
    const patchesDir = path_1.default.join(lockfileDir, patchesDirName);
    const patchedPkgManifest = await (0, read_package_json_1.readPackageJsonFromDir)(userDir);
    const editDir = path_1.default.resolve(opts.dir, userDir);
    const stateValue = (0, stateFile_1.readEditDirState)({
        editDir,
        modulesDir: path_1.default.join(opts.dir, opts.modulesDir ?? 'node_modules'),
    });
    if (!stateValue) {
        throw new error_1.PnpmError('INVALID_PATCH_DIR', `${userDir} is not a valid patch directory`, {
            hint: 'A valid patch directory should be created by `pnpm patch`',
        });
    }
    const { applyToAll } = stateValue;
    const nameAndVersion = `${patchedPkgManifest.name}@${patchedPkgManifest.version}`;
    const patchKey = applyToAll ? patchedPkgManifest.name : nameAndVersion;
    let gitTarballUrl;
    if (!applyToAll) {
        gitTarballUrl = await getGitTarballUrlFromLockfile({
            alias: patchedPkgManifest.name,
            pref: patchedPkgManifest.version,
        }, {
            lockfileDir,
            modulesDir: opts.modulesDir,
            virtualStoreDir: opts.virtualStoreDir,
        });
    }
    const srcDir = tempy_1.default.directory();
    await (0, writePackage_1.writePackage)((0, parse_wanted_dependency_1.parseWantedDependency)(gitTarballUrl ? `${patchedPkgManifest.name}@${gitTarballUrl}` : nameAndVersion), srcDir, opts);
    const patchedPkgDir = await preparePkgFilesForDiff(userDir);
    const patchContent = await diffFolders(srcDir, patchedPkgDir);
    if (patchedPkgDir !== userDir) {
        fs_1.default.rmSync(patchedPkgDir, { recursive: true });
    }
    if (!patchContent.length) {
        return `No changes were found to the following directory: ${userDir}`;
    }
    await fs_1.default.promises.mkdir(patchesDir, { recursive: true });
    const patchFileName = patchKey.replace('/', '__');
    await fs_1.default.promises.writeFile(path_1.default.join(patchesDir, `${patchFileName}.patch`), patchContent, 'utf8');
    const { writeProjectManifest, manifest } = await (0, read_project_manifest_1.tryReadProjectManifest)(lockfileDir);
    const rootProjectManifest = (!opts.sharedWorkspaceLockfile ? manifest : (opts.rootProjectManifest ?? manifest)) ?? {};
    if (!rootProjectManifest.pnpm) {
        rootProjectManifest.pnpm = {
            patchedDependencies: {},
        };
    }
    else if (!rootProjectManifest.pnpm.patchedDependencies) {
        rootProjectManifest.pnpm.patchedDependencies = {};
    }
    rootProjectManifest.pnpm.patchedDependencies[patchKey] = `${patchesDirName}/${patchFileName}.patch`;
    await writeProjectManifest(rootProjectManifest);
    if (opts?.selectedProjectsGraph?.[lockfileDir]) {
        opts.selectedProjectsGraph[lockfileDir].package.manifest = rootProjectManifest;
    }
    if (opts?.allProjectsGraph?.[lockfileDir].package.manifest) {
        opts.allProjectsGraph[lockfileDir].package.manifest = rootProjectManifest;
    }
    return plugin_commands_installation_1.install.handler({
        ...opts,
        rootProjectManifest,
        rawLocalConfig: {
            ...opts.rawLocalConfig,
            'frozen-lockfile': false,
        },
    });
}
async function diffFolders(folderA, folderB) {
    const folderAN = folderA.replace(/\\/g, '/');
    const folderBN = folderB.replace(/\\/g, '/');
    let stdout;
    let stderr;
    try {
        const result = await (0, safe_execa_1.default)('git', ['-c', 'core.safecrlf=false', 'diff', '--src-prefix=a/', '--dst-prefix=b/', '--ignore-cr-at-eol', '--irreversible-delete', '--full-index', '--no-index', '--text', '--no-ext-diff', folderAN, folderBN], {
            cwd: process.cwd(),
            env: {
                ...process.env,
                // #region Predictable output
                // These variables aim to ignore the global git config so we get predictable output
                // https://git-scm.com/docs/git#Documentation/git.txt-codeGITCONFIGNOSYSTEMcode
                GIT_CONFIG_NOSYSTEM: '1',
                HOME: '',
                XDG_CONFIG_HOME: '',
                USERPROFILE: '',
                // #endregion
            },
            stripFinalNewline: false,
        });
        stdout = result.stdout;
        stderr = result.stderr;
    }
    catch (err) { // eslint-disable-line
        stdout = err.stdout;
        stderr = err.stderr;
    }
    // we cannot rely on exit code, because --no-index implies --exit-code
    // i.e. git diff will exit with 1 if there were differences
    if (stderr.length > 0)
        throw new Error(`Unable to diff directories. Make sure you have a recent version of 'git' available in PATH.\nThe following error was reported by 'git':\n${stderr}`);
    return stdout
        .replace(new RegExp(`(a|b)(${(0, escape_string_regexp_1.default)(`/${removeTrailingAndLeadingSlash(folderAN)}/`)})`, 'g'), '$1/')
        .replace(new RegExp(`(a|b)${(0, escape_string_regexp_1.default)(`/${removeTrailingAndLeadingSlash(folderBN)}/`)}`, 'g'), '$1/')
        .replace(new RegExp((0, escape_string_regexp_1.default)(`${folderAN}/`), 'g'), '')
        .replace(new RegExp((0, escape_string_regexp_1.default)(`${folderBN}/`), 'g'), '')
        .replace(/\n\\ No newline at end of file\n$/, '\n');
}
function removeTrailingAndLeadingSlash(p) {
    if (p[0] === '/' || p.endsWith('/')) {
        return p.replace(/^\/|\/$/g, '');
    }
    return p;
}
/**
 * Link files from the source directory to a new temporary directory,
 * but only if not all files in the source directory should be included in the package.
 * If all files should be included, return the original source directory without creating any links.
 * This is required in order for the diff to not include files that are not part of the package.
 */
async function preparePkgFilesForDiff(src) {
    const files = Array.from(new Set((await (0, fs_packlist_1.packlist)(src)).map((f) => path_1.default.join(f))));
    // If there are no extra files in the source directories, then there is no reason
    // to copy.
    if (await areAllFilesInPkg(files, src)) {
        return src;
    }
    const dest = `${src}_tmp`;
    await (0, make_empty_dir_1.default)(dest);
    await Promise.all(files.map(async (file) => {
        const srcFile = path_1.default.join(src, file);
        const destFile = path_1.default.join(dest, file);
        const destDir = path_1.default.dirname(destFile);
        await fs_1.default.promises.mkdir(destDir, { recursive: true });
        await fs_1.default.promises.link(srcFile, destFile);
    }));
    return dest;
}
async function areAllFilesInPkg(files, basePath) {
    const allFiles = await (0, fast_glob_1.default)('**', {
        cwd: basePath,
    });
    return (0, equals_1.default)(allFiles.sort(), files.sort());
}
async function getGitTarballUrlFromLockfile(dep, opts) {
    const { preferredVersions } = await (0, getPatchedDependency_1.getVersionsFromLockfile)(dep, opts);
    return preferredVersions[0]?.gitTarballUrl;
}
//# sourceMappingURL=patchCommit.js.map